
%define ntpservers 131.225.8.127 131.225.17.127

# these macros are used below to help make future stuff
# easy to port from one thing to another
%define service ntpd
%define site fermi
%define firewallservice %{service}.%{site}

#
Summary: This will configure the ntpd config files for Fermilab. 
Name: zz_ntp_configure
Version: 4.2.6
Release: 4
License: GPL
Group: Fermi
URL: http://fermilinux.fnal.gov/
Vendor: Fermilab
Packager: Pat Riehecky
Requires: ntp, ntpdate, coreutils, perl, /bin/awk, chkconfig, /bin/sh
Source0: lsb_release
Source1: %{firewallservice}.slf5
Source2: %{firewallservice}.slf6
BuildArch: noarch

%description

This package configures the ntp time daemon correctly for use at Fermilab.  It
sets the initial time from the default time servers and configures ntpd to
listen to time broadcasts from the local router of the subdomain that the
machine is on.

At this time the default time servers are listed as:
%{ntpservers}

%prep
rm -rf $RPM_BUILD_ROOT

%install
mkdir -p $RPM_BUILD_ROOT/usr/share/doc/%{name}-%{version}/

%files
%defattr(755,root,root)
%doc $RPM_SOURCE_DIR/lsb_release
%doc $RPM_SOURCE_DIR/ntpd.fermi.slf5
%doc $RPM_SOURCE_DIR/ntpd.fermi.slf6

%post
# This copies in the correct firewall script for your version of SLF
VER=$(/usr/share/doc/%{name}-%{version}/lsb_release -r | awk '{ print $2}' | cut -d'.' -f1)
cp -f /usr/share/doc/%{name}-%{version}/%{firewallservice}.slf$VER /etc/sysconfig/%{firewallservice}

%postun
rm -f /etc/sysconfig/%{firewallservice}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -- ntp
if [[ ! -f /etc/sysconfig/%{firewallservice} ]]; then
    # This copies in the correct firewall script for your version of SLF
    VER=$(/usr/share/doc/%{name}-%{version}/lsb_release -r | awk '{ print $2}' | cut -d'.' -f1)
    cp -f /usr/share/doc/%{name}-%{version}/%{firewallservice}.slf$VER /etc/sysconfig/%{firewallservice}
fi

#
# This modifies /etc/init.d/%{service} to poke holes in the firewall for it
#
if [ "$(grep /etc/sysconfig/%{firewallservice} /etc/init.d/%{service})" = "" ] ; then
  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} start
  cat /etc/init.d/%{service} | sed 's:\(start().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} start\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} stop
  cat /etc/init.d/%{service} | sed 's:\(stop().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} stop\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  chmod 755 /etc/init.d/%{service}
fi

#
# This modifies /etc/ntp.conf
#
if [ ! -f /etc/ntp.conf.rpmsave ]; then   # make a backup before changes
    cp -fa /etc/ntp.conf /etc/ntp.conf.rpmsave
fi

if [ `grep "restrict 131.225.0.0 mask 255.255.0.0 nomodify" /etc/ntp.conf | wc -l` -eq 0 ]; then
    #
    #now actually do the modifications
    #
        # this is the legacy method for enabling broadcast client, don't need it
	perl -pe 's/(^enable\s+bclient)//;' -i /etc/ntp.conf 

        # we don't have ntp authentication setup right now
	perl -pe 's/^authenticate\s+(.*)/authenticate no/;' -i /etc/ntp.conf

        # don't use TUV's servers
	perl -pe 's/^server\s+([0-9]+\.rhel\.pool\.ntp\.org)/#server \1/;' -i /etc/ntp.conf

        # we want honesty rather than calculated expectations
        perl -pe 's/^fudge/#fudge/;' -i /etc/ntp.conf

        # no multicast domain to configure so don't use it
	perl -pe 's/^multicastclient/#multicastclient/;' -i /etc/ntp.conf

        # can only have this option on once or things break, disabling it now, enabling later
        # so that no on sets any weird options on it initially
	perl -pe 's/^broadcastclient/#broadcastclient/;' -i /etc/ntp.conf

        # restrict the ntp server's options
	echo "restrict 131.225.0.0 mask 255.255.0.0 nomodify noquery" >> /etc/ntp.conf

        # broadcast is the prefered method so we don't overburden the NTP servers
	echo "broadcastclient" >> /etc/ntp.conf
fi

# This makes sure the daemon is added to the runlevels 3, 4, and 5
# and restarts the service
/sbin/chkconfig --add ntpd
/sbin/chkconfig --levels 345 ntpd on
/etc/init.d/ntpd restart > /dev/null 2>&1

# give the ntp server a chance to look for broadcast peers
# takes 20 seconds for broadcast peers to show up in peers list
sleep 20

# see how many lines of synced servers, there is always 1 line even
# when there are no synced servers, its a header
SYNCSERVERS=$(echo peers | ntpdc 2>/dev/null|wc -l)  # simple querry for peers
if [[ $SYNCSERVERS -lt 2 ]]; then
    # be really really sure
    sleep 5  # give it a really good chance to catch up
    SYNCSERVERS=$(echo peers | ntpdc 2>/dev/null|wc -l)  # simple querry for peers
    if [[ $SYNCSERVERS -lt 2 ]]; then
        # if we aren't sync'd up to anything log it and add explicit servers
        # once added they will stay there unless something removes them
        logger -p daemon.notice -t %{name} "NTP Broadcast not detected, switching to manual servers."

        # The known good FNAL NTP servers should be added to the end
        for server in %{ntpservers}; do
             # asked not to use iburst
#            echo "server $server iburst" >> /etc/ntp.conf
            echo "server $server" >> /etc/ntp.conf
        done

        # make changes take effect
        /etc/init.d/ntpd restart > /dev/null 2>&1
    fi
fi

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -- ntpdate
#
# This correctly configures /etc/ntp/step-tickers
#

if [ -s /etc/ntp/step-tickers ]; then
    if [ ! -f /etc/ntp/step-tickers.rpmsave ]; then
        cp -fa /etc/ntp/step-tickers /etc/ntp/step-tickers.rpmsave
    fi
fi
# insert the correct ntp servers into step-tickers
echo "%{ntpservers}" > /etc/ntp/step-tickers

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -- firstboot
#
# This makes it so that you don't get asked about ntp on the firstboot
# since it is already setup now.
# 
if [ -s /usr/share/firstboot/modules/date.py ]; then
	rm -f /usr/share/firstboot/modules/date.py*
fi

%changelog
* Fri Mar 16 2012 Pat Riehecky <riehecky@fnal.gov> 4.2.6-4
- switched to a define based ntp server list
- updated ntp server list
- made version independent, now works on SLF5 and SLF6
- added comments to help explain the configuration options

* Thu Jun 16 2011 Connie Sieh <csieh@fnal.gov> 4.2.6-3
- Modified ntpd-fermi code from "RH-Firewall-1-INPUT" to "INPUT" as TUV
  Does not use "RH-Firewall-1-INPUT" anymore. 

* Thu May 26 2011 Tyler Parsons <tparsons@fnal.gov> 4.2.6-2
- added secondary and tertiary time servers to ntp.conf (131.225.17.150 and 131.225.8.120)
- changed references to servers from (131.225.8.200 and 131.225.17.200) to (131.225.17.150 and 131.225.8.120)
 
* Mon Aug 16 2010 Troy Dawson <dawson@fnal.gov> 4.2.6-1
- added ntpdate to the requirements
- Changed the step-tickers section to trigger on ntpdate
- Removed the change that added -A to /etc/sysconfig/ntpd

* Mon Jun 14 2010 Troy Dawson <dawson@fnal.gov> 4.2.0-9
- Added noquery to the restrict line

